@@ -8,7 +8,6 @@ Created on 2020/06/15
 @group : hbshare
 """
 
-import pandas as pd
 import simplejson as json
 from functools import partial
 import requests
@@ -31,7 +30,8 @@ class DataApi:
 
     def query(self, url, method='get', post_data={}):
         headers = {
-            "token": self.__token
+            "token": self.__token,
+            "Referer": 'hbshare'
         }
 
         if method == 'get':
@@ -43,6 +43,16 @@ class DataApi:
 
         return result
 
+    def get(self, url):
+        headers = {
+            "token": self.__token,
+            "Referer": 'hbshare'
+        }
+
+        res = requests.get(url, headers=headers, timeout=self.__timeout)
+
+        return res.text
+
     def __getattr__(self, name):
         return partial(self.query, name)
 
