@@ -10,36 +10,11 @@ from hbshare.quant.load_data import load_funds_data
 pymysql.install_as_MySQLdb()
 
 
-def ret(data_df):
-    funds = data_df.columns.tolist()
-    date_col = 't_date'
-    if 't_date' in funds:
-        funds.remove(date_col)
-    elif '日期' in funds:
-        date_col = '日期'
-        funds.remove('日期')
-
-    result_all = data_df['t_date']
-
-    for i in funds:
-        single_df = data_df[[date_col, i]].copy()
-        single_df['ret'] = single_df[i] / single_df[i].shift(1) - 1
-
-        result_all = pd.merge(result_all, single_df[['t_date', 'ret']], on='t_date').rename(columns={'ret': i})
-
-    return result_all
-
-
 # 计算每只产品的年化收益，年华波动，夏普，最大回撤等指标
 def performance_analysis(
-        data_df,
-        risk_free=0.015,
-        start_date=datetime(2019, 1, 1).date(),
-        end_date=datetime(2099, 12, 31).date(),
-        ret_num_per_year=52,
-        monthly_funds=None
+        data_df, risk_free=0.015, start_date=datetime(2019, 1, 1).date(), end_date=datetime.now().date(),
+        ret_num_per_year=52, monthly_funds=None
 ):
-
     if monthly_funds is None:
         monthly_funds = []
     funds = data_df.columns.tolist()
@@ -162,8 +137,7 @@ def performance_analysis(
     return result_all, dd_all, ret_all
 
 
-# 计算每只产品每月收益
-def performance_monthly_ret(data_df):
+def ret(data_df):
     funds = data_df.columns.tolist()
     date_col = 't_date'
     if 't_date' in funds:
@@ -171,18 +145,30 @@ def performance_monthly_ret(data_df):
     elif '日期' in funds:
         date_col = '日期'
         funds.remove('日期')
-
-    result_all = pd.DataFrame(
-        {
-            't_date': data_df[date_col][1:]
-        }
-    )
-
+    result_all = data_df['t_date']
     for i in funds:
-        ret = data_df[i] / data_df[i].shift(1) - 1
-        result_all[i] = ret[1:]
+        single_df = data_df[[date_col, i]].copy()
+        single_df['ret'] = single_df[i] / single_df[i].shift(1) - 1
 
-    return result_all.iloc[::-1]
+        result_all = pd.merge(result_all, single_df[['t_date', 'ret']], on='t_date').rename(columns={'ret': i})
+    return result_all
+
+
+# 计算每只产品每月收益
+def performance_monthly_ret(data_df):
+    # funds = data_df.columns.tolist()
+    # date_col = 't_date'
+    # if 't_date' in funds:
+    #     funds.remove(date_col)
+    # elif '日期' in funds:
+    #     date_col = '日期'
+    #     funds.remove('日期')
+    # result_all = pd.DataFrame({'t_date': data_df[date_col][1:]})
+    # for i in funds:
+    #     ret = data_df[i] / data_df[i].shift(1) - 1
+    #     result_all[i] = ret[1:]
+    result_all = ret(data_df=data_df)
+    return result_all[1:].iloc[::-1]
 
 
 # 按类型计算产品平均收益率
@@ -229,6 +215,7 @@ def performance_specific_ret(data_df, form=True, monthly_funds=None):
     ret_last_week = []
     ret_this_4 = []
     ret_this_8 = []
+    ret_6_months = []
     ret_2021 = []
     ret_2020 = []  # 年化
     ret_2019 = []  # 年化
@@ -243,11 +230,13 @@ def performance_specific_ret(data_df, form=True, monthly_funds=None):
             ret_last_week.append(single_df[i][len(single_df) - 2] / single_df[i][len(single_df) - 3] - 1)
             ret_this_4.append(single_df[i][len(single_df) - 1] / single_df[i][len(single_df) - 5] - 1)
             ret_this_8.append(single_df[i][len(single_df) - 1] / single_df[i][len(single_df) - 9] - 1)
+            ret_6_months.append(single_df[i][len(single_df) - 1] / single_df[i][len(single_df) - 25] - 1)
         else:
             ret_this_week.append(None)
             ret_last_week.append(None)
             ret_this_4.append(None)
             ret_this_8.append(None)
+            ret_6_months.append(None)
 
         nav_2018 = single_df[
             np.array(single_df[date_col] < datetime(2019, 1, 1).date())
@@ -341,6 +330,7 @@ def performance_specific_ret(data_df, form=True, monthly_funds=None):
                 '上周收益': ret_last_week,
                 '近四周收益': ret_this_4,
                 '近八周收益': ret_this_8,
+                '近六月收益': ret_6_months,
                 '2021累计': ret_2021,
                 '2020收益': ret_2020,
                 '2019收益': ret_2019,
