@@ -1,4 +1,5 @@
 from hbshare.quant.load_data import load_funds_data, load_funds_alpha
+from hbshare.quant.fund_perf import ret
 import pandas as pd
 import numpy as np
 import pyecharts.options as opts
@@ -102,7 +103,7 @@ def nav_lines(
 
 
 def gen_grid(
-        end_date, funds, zz=False, lookback_years=3, grid_width=1200, grid_height=500, axis_cross=None,
+        end_date, funds, zz=False, lookback_years=3, grid_width=1200, grid_height=500, pos_top=23, axis_cross=None,
         db_path=sql_write_path_hb['work'], cal_path=sql_write_path_hb['daily'], all_selected=True
 ):
     if axis_cross:
@@ -155,7 +156,7 @@ def gen_grid(
                 pos_top="7%"
             )
         ),
-        grid_opts=opts.GridOpts(pos_left='55%', pos_top="23%")
+        grid_opts=opts.GridOpts(pos_left='55%', pos_top=str(pos_top) + "%")
     ).add(
         web2.set_global_opts(
             title_opts=opts.TitleOpts(
@@ -169,7 +170,7 @@ def gen_grid(
                 pos_top="7%"
             )
         ),
-        grid_opts=opts.GridOpts(pos_right='55%', pos_top="23%")
+        grid_opts=opts.GridOpts(pos_right='55%', pos_top=str(pos_top) + "%")
     )
     return grid_nav
 
@@ -519,3 +520,147 @@ def chart_fin_fut_amt(product, start_date=None, end_date=None, grid_width=1200,
     )
 
     return grid
+
+
+def chart_freq_dist(
+        funds, end_date, lookback_year, grid_width=1200, grid_height=600, pos_top=20, interval=0.5,
+        dtype='', fillna=False, freq='w',
+        db_path=sql_write_path_hb['work'], cal_path=sql_write_path_hb['daily']
+):
+    date_end = end_date
+    date_start = date_end - timedelta(days=365 * lookback_year)
+    table = 'fund_list'
+
+    fund_list = pd.read_sql_query(
+        'select * from ' + table + ' where `name` in ' + str(tuple(funds)), create_engine(db_path)
+    )
+
+    if dtype.lower() != 'alpha':
+        funds_data = load_funds_data(
+            fund_list=fund_list,
+            first_date=date_start,
+            end_date=date_end,
+            fillna=fillna,
+            db_path=db_path,
+            cal_db_path=cal_path,
+            freq=freq
+        )
+    else:
+        funds_data = load_funds_alpha(
+            fund_list=fund_list,
+            first_date=date_start,
+            end_date=date_end,
+            fillna=fillna,
+            db_path=db_path,
+            cal_db_path=cal_path,
+            freq=freq
+        )
+
+    ret_df = ret(data_df=funds_data)
+    ret_data = ret_df[funds]
+    ret_values = np.reshape(ret_data.values, (1, ret_data.values.shape[0] * ret_data.values.shape[1]))
+    ret_values = ret_values[~np.isnan(ret_values)] * 100
+    ret_floor = np.floor(min(ret_values)) - 1
+    ret_ceil = np.ceil(max(ret_values))
+    ret_space = np.linspace(start=ret_floor, stop=ret_ceil, num=int((ret_ceil - ret_floor) / interval + 1))
+    dist_df = pd.DataFrame({'zone': ret_space})
+    dist_df['label'] = dist_df['zone'].apply(lambda x: str(x) + '% ~ ' + str(x + interval) + '%')
+    for i in funds:
+        dist_df[i] = 0
+        for j in range(1, len(ret_df)):
+            fund_ret = ret_df[i][j] * 100
+            if np.isnan(fund_ret):
+                continue
+            fund_ret_index = sum(ret_space <= fund_ret) - 1
+
+            dist_df.loc[fund_ret_index, i] += 1
+
+    web = Bar(
+    ).add_xaxis(
+        xaxis_data=dist_df['label'].tolist()
+    )
+    for i in funds:
+        web.add_yaxis(
+            series_name=i,
+            y_axis=dist_df[i].tolist(),
+            stack='总量',
+            label_opts=opts.LabelOpts(is_show=False),
+            # areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
+        )
+
+    web.set_global_opts(
+        title_opts=opts.TitleOpts(
+            title='近' + str(lookback_year) + '年产品周收益分布（平均周收益：' + str(np.mean(ret_values).round(2))
+                  + '%）' + funds_data['t_date'].tolist()[-1].strftime('%Y-%m-%d')
+        ),
+        tooltip_opts=opts.TooltipOpts(
+            # trigger="axis", axis_pointer_type="cross"
+        ),
+        yaxis_opts=opts.AxisOpts(
+            type_="value",
+            axislabel_opts=opts.LabelOpts(formatter="{value}"),
+            axistick_opts=opts.AxisTickOpts(is_show=True),
+            splitline_opts=opts.SplitLineOpts(is_show=True),
+        ),
+        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
+        # datazoom_opts=[
+        #     opts.DataZoomOpts(
+        #         range_start=0,
+        #         range_end=100,
+        #         # is_show=False,
+        #         type_="inside",
+        #         # xaxis_index=[0, 1],
+        #     ),
+        #     opts.DataZoomOpts(
+        #         #     is_show=True,
+        #         #     xaxis_index=[0, 1],
+        #         #     type_="slider",
+        #         #     pos_top="90%",
+        #         #     range_start=98,
+        #         #     range_end=100,
+        #     ),
+        # ],
+        legend_opts=opts.LegendOpts(
+            pos_top="7%",
+            pos_left='5%',
+            pos_right='5%'
+        )
+    )
+    # web2.set_global_opts(
+    #     legend_opts=opts.LegendOpts(
+    #         pos_top="7%",
+    #         pos_left='10%',
+    #         pos_right='20%'
+    #     )
+    # )
+    grid = (
+        Grid(
+            init_opts=opts.InitOpts(
+                width=str(grid_width) + "px",
+                height=str(grid_height) + "px"
+            ),
+        ).add(
+            web, grid_opts=opts.GridOpts(
+                pos_right="10%",
+                pos_left="10%",
+                pos_top=str(pos_top) + '%',
+                # pos_bottom='50%'
+            ),
+            # is_control_axis_index=True
+            # ).add(
+            #     web2, grid_opts=opts.GridOpts(
+            #         pos_right="20%",
+            #         pos_left="10%",
+            #         pos_top='55%',
+            #         pos_bottom='15%'
+            #     ),
+        )
+    )
+    return grid
+
+
+# def chart_ret_with_index(
+#         funds, end_date, lookback_year, grid_width=1200, grid_height=600, interval=0.5,
+#         dtype='', fillna=False, freq='w',
+#         db_path=sql_write_path_hb['work'], cal_path=sql_write_path_hb['daily'], *kwargs
+# )
